from .enums import *
from .response import OppoResponse
from .response import OppoIntResponse
from .response import OppoStringResponse
from .response import OppoTimeResponse
from .response import OppoPowerResponse
from .response import OppoPlayResponse
from .response import OppoDiscTypeResponse
from .response import OppoHdmiModeResponse
from .response import OppoHdrSettingResponse
from .response import OppoInputSourceResponse
from .response import OppoRepeatModeResponse
from .response import OppoSpeedModeResponse
from .response import OppoVideoHdrStatusResponse
from .response import OppoVideo3dStatusResponse
from .response import OppoZoomModeResponse
from .response import OppoUpdatePowerStatusResponse
from .response import OppoUpdatePlayStatusResponse
from .response import OppoUpdateTimeResponse
from .factory import get_response