from .enums import *
from .command import OppoCommand

from .query_command import OppoQueryCommand
from .query_command import OppoQueryCdCommand
from .query_command import OppoQueryDirectoryCommand

from .remote_command import OppoRemoteCommand

from .set_command import OppoSetCommand
from .set_command import OppoSetVerboseModeCommand
from .set_command import OppoSetHdmiModeCommand
from .set_command import OppoSetZoomModeCommand
from .set_command import OppoSetVolumeLevelCommand
from .set_command import OppoSetRepeatModeCommand
from .set_command import OppoSetSubtitleShiftCommand
from .set_command import OppoSetOsdPositionCommand
from .set_command import OppoSetTimeCodeModeCommand
from .set_command import OppoSetHdrModeCommand
from .set_command import OppoSetInputSourceCommand
from .set_command import OppoSetScreensaverCommand
from .set_command import OppoSetAppModeCommand
from .set_command import OppoSetSacdPriorityCommand
from .set_command import OppoSetSacdOutputModeCommand
from .set_command import OppoSetFwdModeCommand
from .set_command import OppoSetRevModeCommand
from .set_command import OppoSetTitleCommand
from .set_command import OppoSetChapterCommand
from .set_command import OppoSetChapterPositionCommand
from .set_command import OppoSetTitlePositionCommand

